@@ -1,5 +1,5 @@
 #
-# Copyright John Reid 2009, 2010, 2011, 2012
+# Copyright John Reid 2009, 2010, 2011, 2012, 2013
 #
 
 from __future__ import with_statement
@@ -41,6 +41,7 @@ import time
 import subprocess
 import pylab as P
 import warnings
+import cPickle
 from optparse import OptionParser, OptionGroup
 from itertools import imap
 from collections import defaultdict
@@ -324,6 +325,11 @@ def add_options(option_parser):
         "Internal",
         "Not normally used."
     )
+    internal_options.add_option(
+        "--cache-index",
+        action='store_true',
+        help="Save the index to disk so it does not need to be rebuilt next time."
+    )
     internal_options.add_option(
         "-Q",
         dest="pvalue_table_lattice_size",
@@ -821,14 +827,39 @@ def _read_fasta(fasta):
 
 
 def read_sequences(fasta_file, options):
-    """Read sequences and builds index.
-    
-    Also sets minimum and maximum # of sites if not already in options.
+    """Read sequences and builds index. Will save index if options.save_index
+    is set. If saved index is present will load the index instead of re-reading
+    sequences and building the index.
     """
-    num_bases, seqs, ids = read_fasta(fasta_file)
-    logger.info('Read %d bases from %d sequences in %s',
-                num_bases, len(seqs), fasta_file)
-    index = build_index(seqs)
+    index_file = fasta_file + '.esa'
+    info_file = fasta_file + '.info'
+    successful_load = False  # Did we load the index successfully?
+    #if True:
+    try:
+        logger.info('Looking for cached index information: %s', info_file)
+        num_bases, ids = cPickle.load(open(info_file))
+        logger.info('Loading index of %d bases from %s.*',
+                    num_bases, index_file)
+        index = load_index(index_file)
+        seqs = index.text()
+        successful_load = True  # Did we load the index successfully?
+    except:
+        logger.info('Could not load cached index, reading FASTA directly')
+        num_bases, seqs, ids = read_fasta(fasta_file)
+        logger.info('Read %d bases from %d sequences in %s',
+                    num_bases, len(seqs), fasta_file)
+        logger.info('Initialising index')
+        index = build_index(seqs)
+        try:  # put in try-clause as not all options may have it
+            if options.cache_index:
+                logger.info('Creating index')
+                index.visit(max_depth=options.max_w)
+                logger.info('Saving index info to %s', info_file)
+                cPickle.dump((num_bases, ids), open(info_file, 'w'))
+                logger.info('Saving index to %s', index_file)
+                save_index(index, index_file)
+        except AttributeError:
+            warnings.warn('Options may not have save_index attribute')
     return num_bases, seqs, ids, index
 
 
@@ -1212,7 +1243,7 @@ class StartingPointFinder(object):
         """
         Add a callback to the start finder that controls how long it spends executing.
         This callback times how long each start takes to evaluate and changes the number that
-        are ignored in order to finish on time. 
+        are ignored in order to finish on time.
         """
         total_to_examine = start_finder.num_starts
         logger.info(
@@ -1656,7 +1687,7 @@ class MotifFinder(object):
     def __call__(self, starting_point_finder):
         """
         Find the instances by running EM on each start.
-        
+
         Takes a list of instances already found that is updated.
         """
         #
@@ -2001,6 +2032,7 @@ class SequenceSet(object):
         self.options = options
         self.num_bases, self.seqs, self.ids, self.index = read_sequences(
             fasta_filename, self.options)
+        logging.info('max_W=%d', self.options.max_w)
         self.data = Data(self.index, max_W=self.options.max_w)
         self.occs = occurrences_from_index(self.data.index)
         self.freqs = ZeroOrderFrequencies(list(self.occs[:4]))
